'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

let sortScopes = (() => {
  var _ref = _asyncToGenerator(function* (scopes, terms, executor) {
    (0, _debug2.default)("sortScopes()", scopes, terms, executor);
    terms = terms.slice();
    // Map each object into terms sorted by cardinality
    // so [{a: 1, b: "a"}, {a: 2, b: "b"}]|order(a, b)
    // becomes [[1, "a"], [2, "b"]]
    const itemWithTerms = [];
    (0, _debug2.default)('scopes to be sorted', scopes);
    for (let i = 0; i < scopes.length; i++) {
      const scope = scopes[i];
      const termValues = yield Promise.all(terms.map(function (term) {
        return executor.exec(term.expression, scope);
      })).then(function (scopes) {
        return scopes.map(function (scope) {
          return scope.value;
        });
      });
      itemWithTerms.push({
        scope,
        terms: termValues
      });
    }
    // const itemWithTerms = scopes.map(async scope => ({
    //   scope: scope,
    //   terms: Promise.all(terms.map(term => {
    //     return executor.exec(term.expression, scope)
    //   }))
    // }))
    (0, _debug2.default)('itemWithTerms', itemWithTerms);
    const termIsAscending = terms.map(function (term) {
      return term.direction == 'asc';
    });
    (0, _debug2.default)('term is ascending', termIsAscending);

    const sorted = itemWithTerms.sort(function (a, b) {
      for (let i = 0; i < terms.length; i++) {
        const ascending = termIsAscending[i];
        const comparision = compare(a.terms[i], b.terms[i], ascending);
        if (comparision !== 0) {
          return comparision;
        }
      }
      return 0;
    });

    (0, _debug2.default)('sorted', sorted);

    return sorted.map(function (item) {
      return item.scope;
    });
  });

  return function sortScopes(_x, _x2, _x3) {
    return _ref.apply(this, arguments);
  };
})();

var _Scope = require('./Scope');

var _Scope2 = _interopRequireDefault(_Scope);

var _debug = require('../debug');

var _debug2 = _interopRequireDefault(_debug);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { return Promise.resolve(value).then(function (value) { step("next", value); }, function (err) { step("throw", err); }); } } return step("next"); }); }; }

// Coerce a value to be on the comparable form. E.g. rewrites path("a.b.c") => "a.b.c"
function coerce(value) {
  if (value === null || value === undefined) {
    return null;
  }
  if (value.toJSON) {
    return value.toJSON();
  }
  return value;
}

const TYPE_ORDER = {
  'number': 1,
  'string': 2,
  'boolean': 3,
  'object': 4
};

function compare(a, b, ascending) {
  if (a === b) {
    return 0;
  }
  if (typeof a !== typeof b) {
    if (TYPE_ORDER[typeof a] < TYPE_ORDER[typeof b]) {
      return -1;
    }
    return 1;
  }
  if (a < b) {
    return ascending ? -1 : 1;
  }
  return ascending ? 1 : -1;
}

exports.default = (() => {
  var _ref2 = _asyncToGenerator(function* (input, terms, executor) {
    (0, _debug2.default)("sort()", input, terms, executor);
    if (Array.isArray(input)) {
      // Input is array of scopes
      (0, _debug2.default)("Sorting as scopes");
      return sortScopes(input, terms, executor);
    }
    if (Array.isArray(input.value)) {
      (0, _debug2.default)("Sorting array by unboxing");
      // Input is scoped array
      const scoped = input.value.map(function (item) {
        return input.clone({
          value: item
        });
      });
      return sortScopes(scoped, terms, executor);
      // Should probably rebox output so that `*|order(ordinal).ordinal` was illegal. Should only
      // be [] that performs unboxing. But we have tests documenting the non-reboxing behavior, so
      // we'll keep it for now. If we re-box you would have to do this instead: `*|order(ordinal)[].ordinal`
      // // Re-box sorted output
      // return input.clone({
      //   value: sorted.map(scope => scope.value)
      // })
    }
    // Item is a plain value, just pass it through
    (0, _debug2.default)("Plain value, sorting by pass-through, but in an array of scopes");
    return [input];
  });

  function sort(_x4, _x5, _x6) {
    return _ref2.apply(this, arguments);
  }

  return sort;
})();
//# sourceMappingURL=sort.js.map